/*
	XML simple read example

	inhak.min@gmail.com
*/

#include <scew/scew.h>
#include <stdio.h>

char *get_attribute(scew_element *node, char *name)
{
	if (node != NULL)
	{
		scew_list *list = scew_element_attributes (node);
		while (list != NULL)
		{
			scew_attribute *attribute = scew_list_data (list);
			if ( !strcmp(scew_attribute_name (attribute), name) )
				return (char *)scew_attribute_value (attribute);

			list = scew_list_next (list);
		}
	}
}

char *get_node_path(scew_element *node)
{
	static char result_path[9072];

	scew_element *parent = node;

	sprintf(result_path, "");
	
	while(parent)
	{
		char *name = (char *)scew_element_name(parent);

		char tmp[9072];
		sprintf(tmp, "%s", result_path);

		sprintf(result_path, "%s", "/");
		sprintf(result_path, "%s%s", result_path, name);
		sprintf(result_path, "%s%s", result_path, tmp);

		parent = scew_element_parent(parent);
	}

	return result_path;
}

int get_node_depth(scew_element *node)
{
	int depth=0;
	scew_element *parent = node;

	while(parent)
	{
		++depth;
		parent = scew_element_parent(parent);
	}

	return depth;
}

scew_list* find_nodes (scew_element *node, char *find_path)
{
	static scew_list *result_list=NULL;

	char *path = get_node_path(node);
	if ( !strcmp(find_path, path) || !(strcmp(find_path, "*")) )
	{
		if(result_list==NULL)
			result_list = scew_list_create(node);
		else
			scew_list_append(result_list, node);
	}

	scew_list *children_list = scew_element_children (node);
	while (children_list != NULL)
	{
		scew_element *child = scew_list_data (children_list);
		result_list = find_nodes (child, find_path);
		children_list = scew_list_next (children_list);
	}

	return result_list;
}

static void
print_attributes (scew_element *node)
{
	if (node != NULL)
	{
		scew_list *list = scew_element_attributes (node);
		while (list != NULL)
		{
			scew_attribute *attribute = scew_list_data (list);
			printf (("%s=\"%s\""),
				scew_attribute_name (attribute),
				scew_attribute_value (attribute));
			list = scew_list_next (list);
		}
	}
}

//   
void demo(scew_tree *xml)
{
	scew_list *node_list=NULL;

	char *path = "/ctml/speciesData/species/thermo/NASA/floatArray";
	//char *path = "*";
	node_list = find_nodes (scew_tree_root (xml), path);

	printf(">> Found nodes: %d\n\n", scew_list_size(node_list));

	// Ʈ ˻ؼ contents  ̾Ƴ.
	while (node_list != NULL)
	{
		scew_element *node = scew_list_data (node_list);

		printf("[%d] %s\n", get_node_depth(node), get_node_path(node));

		// contents  Ѵ.
		char *contents = (char *)scew_element_contents (node);
		printf("contents: %s\n\n", contents);
		
		// attribute  Ѵ.
		printf("attributes: ");
		print_attributes (node);

		printf("\n-------------------\n");

		//  带 ´.
		node_list = scew_list_next (node_list);
	}

	scew_list_free(node_list);
}

int
main (int argc, char *argv[])
{
	scew_reader *reader = NULL;
	scew_parser *parser = NULL;
	scew_tree *xml = NULL;

	if (argc < 2)
	{
		printf ("Usage: %s file.xml\n", argv[0]);
		return -1;
	}

	/* Creates an SCEW parser. This is the first function to call. */
	parser = scew_parser_create ();

	scew_parser_ignore_whitespaces (parser, SCEW_TRUE);

	/* Loads an XML file. */
	reader = scew_reader_file_create (argv[1]);

	xml = scew_parser_load (parser, reader);

	demo(xml);

	/* Remember to free xml (scew_parser_free does not free it). */
	scew_tree_free (xml);

	/* Frees the SCEW parser and reader. */
	scew_reader_free (reader);
	scew_parser_free (parser);

	return 0;
}
